/* 	Copyright John D. "smeezekitty" 2017
	
	FreeNBTV is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FreeNBTV is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FreeNBTV.  If not, see <http://www.gnu.org/licenses/>.
*/

/* This file handles the channel setup GUI */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include "nbtv_format.h"
#include "input.h"
#include "nbtv_gen.h"
#include "gui.h"

const char *video_colors[NUM_VIDEO_COLORS] = {"0 MONO", "1 RED", "2 GREEN", "3 BLUE", "4 CYAN", "5 MAGENTA", "6 YELLOW", "7 ORANGE", "8 WHITE"};

GtkWidget *cs_window;
GtkWidget *tabby;
struct cs_tab {
	GtkWidget *main_grid;
	GtkWidget *selector_grid;
	GtkWidget *video_grid;
	GtkWidget *audio_grid;
	GtkWidget *sync_grid;
	GtkWidget *selector_frame;
	GtkWidget *channel_gain;
	GtkWidget *channel_invert;
	GtkWidget *channel_gain_frame;
	GtkWidget *channel_gain_grid;
	GtkWidget *video_frame;
	GtkWidget *audio_frame;
	GtkWidget *sync_frame;
	GtkWidget *audio_cb;
	GtkWidget *video_cb;
	GtkWidget *sync_cb;
	GtkWidget *invert_video;
	GtkWidget *video_gain;
	GtkWidget *video_color;
	GtkWidget *Rg,*Gg,*Bg;
	GtkWidget *video_dcoff;
	GtkWidget *invert_audio;
	GtkWidget *audio_gain;
	GtkWidget *audio_dcoff;
	GtkWidget *audio_agc;
	GtkWidget *sync_dcoff;
	GtkWidget *line_sync;
	GtkWidget *pos_ls;
	GtkWidget *ls_pulse_len;
	GtkWidget *ls_pulse_amp;
	GtkWidget *frame_sync;
	GtkWidget *fs_oll, *fs_ofl;
	GtkWidget *fs_pulse, *fs_pulse_pos;
	GtkWidget *fs_pulse_len;
	GtkWidget *fs_pulse_amp;
} cs_tabs[4];

struct cs_tab *tab;

char c_handler_disable; //Disable the callback handlers

void cs_window_close(GtkWidget *w, gpointer bla){
	gtk_widget_destroy(cs_window);
	cs_window = NULL;
}
GtkWidget *create_video_color_combobox(){
	GtkWidget *cb = gtk_combo_box_text_new();
	int i;
	for(i = 0;i < NUM_VIDEO_COLORS;i++){
		gtk_combo_box_text_append(GTK_COMBO_BOX_TEXT(cb), NULL, video_colors[i]);
	}
	gtk_combo_box_set_active(GTK_COMBO_BOX(cb), 0);
	return cb;
}
/*Update the parameters to match the internal state*/
void gui_update_channel_setup(struct NBTV_format *fmt){
	int i;
	struct NBTV_channel *ch;
	if(!cs_window) return;
	c_handler_disable = 1;
	for(i = 0;i < 4;i++){
		ch = &(fmt->channels)[i];
		tab = &cs_tabs[i];
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tab->channel_invert), ch->invert_channel);
		gtk_range_set_value(GTK_RANGE(tab->channel_gain), ch->channel_gain);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tab->video_cb), ch->video);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tab->audio_cb), ch->audio);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tab->sync_cb), ch->sync);
		gtk_widget_set_sensitive(tab->video_frame, ch->video);
		gtk_widget_set_sensitive(tab->audio_frame, ch->audio);
		gtk_widget_set_sensitive(tab->sync_frame, ch->sync);
		gtk_widget_set_sensitive(tab->video_cb, !ch->audio);
		gtk_widget_set_sensitive(tab->sync_cb, !ch->audio);
		gtk_widget_set_sensitive(tab->audio_cb, /*!(ch->video || ch->sync)*/0); // Audio not implemented yet
		if(ch->video){
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tab->invert_video), ch->invert_video);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(tab->video_gain), ch->video_level);
			gtk_combo_box_set_active(GTK_COMBO_BOX(tab->video_color), ch->video_color);
			gtk_range_set_value(GTK_RANGE(tab->Rg), ch->red_gain);
			gtk_range_set_value(GTK_RANGE(tab->Gg), ch->blue_gain);
			gtk_range_set_value(GTK_RANGE(tab->Bg), ch->blue_gain);
			/***************************************************/
			gtk_widget_set_sensitive(tab->Rg, 0);
			gtk_widget_set_sensitive(tab->Gg, 0);
			gtk_widget_set_sensitive(tab->Bg, 0);
			gtk_widget_set_sensitive(tab->video_color, 0);
			/***************************************************/
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(tab->video_dcoff), ch->video_dc_off);

		}
		if(ch->audio){
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tab->invert_audio), ch->invert_audio);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tab->audio_agc), ch->audio_agc);
			gtk_range_set_value(GTK_RANGE(tab->audio_gain), ch->audio_level);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(tab->audio_dcoff), ch->audio_dc_off);
		}
		if(ch->sync){
			char buffer[8];
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tab->line_sync), ch->line_sync);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tab->pos_ls), ch->pos_line_sync);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tab->frame_sync), ch->frame_sync);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tab->fs_pulse), ch->fs_pulse);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tab->fs_pulse_pos), ch->pos_frame_sync);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tab->fs_ofl), ch->fs_no_ls_f);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tab->fs_oll), ch->fs_no_ls_l);
			gtk_widget_set_sensitive(tab->fs_pulse, ch->frame_sync);
			gtk_widget_set_sensitive(tab->fs_pulse_pos, ch->frame_sync && ch->fs_pulse);
			gtk_widget_set_sensitive(tab->fs_pulse_amp, ch->frame_sync && ch->fs_pulse);
			gtk_widget_set_sensitive(tab->fs_pulse_len, ch->frame_sync && ch->fs_pulse && !ch->video);
			gtk_widget_set_sensitive(tab->fs_oll, ch->frame_sync && ch->line_sync);
			gtk_widget_set_sensitive(tab->fs_ofl, ch->frame_sync && ch->line_sync);
			gtk_entry_set_text(GTK_ENTRY(tab->ls_pulse_len), gcvt(ch->ls_length, 4, buffer));
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(tab->ls_pulse_amp), ch->ls_level);
			gtk_entry_set_text(GTK_ENTRY(tab->fs_pulse_len), gcvt(ch->fs_length, 4, buffer));
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(tab->fs_pulse_amp), ch->fs_level);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(tab->sync_dcoff), ch->sync_dc_off);
		}
	}
	c_handler_disable = 0;
}
			
void create_selection_panel(struct cs_tab *tab){
	tab->selector_frame = gtk_frame_new("Stream Select: ");
	gtk_grid_attach(GTK_GRID(tab->main_grid),tab->selector_frame,0,0,1,1);
	tab->selector_grid = gtk_grid_new();
	gtk_container_add(GTK_CONTAINER(tab->selector_frame), tab->selector_grid);
	tab->audio_cb = gtk_check_button_new_with_label("Audio");
	tab->video_cb = gtk_check_button_new_with_label("Video");
	tab->sync_cb = gtk_check_button_new_with_label("Sync");
	gtk_grid_attach(GTK_GRID(tab->selector_grid), tab->audio_cb, 0, 0, 1, 1);
	gtk_grid_attach(GTK_GRID(tab->selector_grid), tab->video_cb, 1, 0, 1, 1);
	gtk_grid_attach(GTK_GRID(tab->selector_grid), tab->sync_cb, 2, 0, 1, 1);
	tab->channel_gain_frame = gtk_frame_new("Channel Gain: ");
	tab->channel_gain_grid = gtk_grid_new();
	gtk_grid_set_column_homogeneous(GTK_GRID(tab->channel_gain_grid), TRUE);
	gtk_container_add(GTK_CONTAINER(tab->channel_gain_frame), tab->channel_gain_grid);
	gtk_grid_attach(GTK_GRID(tab->main_grid),tab->channel_gain_frame, 1, 0, 1, 1);
	tab->channel_invert = gtk_check_button_new_with_label("Invert Channel");
	gtk_grid_attach(GTK_GRID(tab->channel_gain_grid), tab->channel_invert, 0, 0, 1, 1);
	tab->channel_gain = gtk_scale_new_with_range(GTK_ORIENTATION_HORIZONTAL, -100, 100, 5.0);
	gtk_scale_set_has_origin(GTK_SCALE(tab->channel_gain), TRUE);
	gtk_grid_attach(GTK_GRID(tab->channel_gain_grid), tab->channel_gain, 1, 0, 1, 1);
}
void create_video_panel(struct cs_tab *tab){
	tab->video_frame = gtk_frame_new("Video Setup: ");
	tab->video_grid = gtk_grid_new();
	gtk_grid_attach(GTK_GRID(tab->main_grid), tab->video_frame, 0, 1, 1, 1);
	gtk_container_add(GTK_CONTAINER(tab->video_frame), tab->video_grid);
	tab->invert_video = gtk_check_button_new_with_label("Invert Video");
	gtk_grid_attach(GTK_GRID(tab->video_grid), tab->invert_video, 0, 0, 1, 1);
	tab->video_gain = gtk_spin_button_new_with_range(0, 100, 1.0);
	put_label(tab->video_grid, "Video Level:\n(% F/S) ", 0, 1);
	gtk_grid_attach(GTK_GRID(tab->video_grid), tab->video_gain, 1, 1, 1, 1);
	put_label(tab->video_grid, "Channel Color: ", 0, 2);
	tab->video_color = create_video_color_combobox();
	gtk_grid_attach(GTK_GRID(tab->video_grid), tab->video_color, 1, 2, 1, 1);
	put_label_colored(tab->video_grid, 192, 0, 0, "Red Gain: ", 0, 3);
	put_label_colored(tab->video_grid, 0, 128, 0, "Green Gain: ", 0, 4);
	put_label_colored(tab->video_grid, 0, 0, 255, "Blue Gain: ", 0, 5);
	tab->Rg = gtk_scale_new_with_range(GTK_ORIENTATION_HORIZONTAL, -100, 100, 5.0);
	tab->Gg = gtk_scale_new_with_range(GTK_ORIENTATION_HORIZONTAL, -100, 100, 5.0);
	tab->Bg = gtk_scale_new_with_range(GTK_ORIENTATION_HORIZONTAL, -100, 100, 5.0);
	gtk_scale_set_has_origin(GTK_SCALE(tab->Rg), TRUE);
	gtk_scale_set_has_origin(GTK_SCALE(tab->Gg), TRUE);
	gtk_scale_set_has_origin(GTK_SCALE(tab->Bg), TRUE);
	gtk_grid_attach(GTK_GRID(tab->video_grid), tab->Rg, 1, 3, 1, 1);
	gtk_grid_attach(GTK_GRID(tab->video_grid), tab->Gg, 1, 4, 1, 1);
	gtk_grid_attach(GTK_GRID(tab->video_grid), tab->Bg, 1, 5, 1, 1);
	put_label(tab->video_grid, "DC Offset (% F/S): ", 0, 6);
	tab->video_dcoff = gtk_spin_button_new_with_range(-100, 100, 1);
	gtk_grid_attach(GTK_GRID(tab->video_grid), tab->video_dcoff, 1, 6, 1, 1);
}
void create_audio_panel(struct cs_tab *tab){
	tab->audio_frame = gtk_frame_new("Audio Setup: ");
	tab->audio_grid = gtk_grid_new();
	gtk_grid_attach(GTK_GRID(tab->main_grid), tab->audio_frame, 0, 2, 1, 1);
	gtk_container_add(GTK_CONTAINER(tab->audio_frame), tab->audio_grid);
	tab->invert_audio = gtk_check_button_new_with_label("Invert Audio");
	gtk_grid_attach(GTK_GRID(tab->audio_grid), tab->invert_audio, 0, 0, 1, 1);
	tab->audio_gain = gtk_scale_new_with_range(GTK_ORIENTATION_HORIZONTAL, 0, 100, 1);
	gtk_scale_set_has_origin(GTK_SCALE(tab->audio_gain), TRUE);
	put_label(tab->audio_grid, "Audio Gain: ", 0, 1);
	put_label(tab->audio_grid, "DC Offset (% F/S): ", 0, 2);
	gtk_grid_attach(GTK_GRID(tab->audio_grid), tab->audio_gain, 1, 1, 1, 1);
	tab->audio_agc = gtk_check_button_new_with_label("AGC");
	gtk_grid_attach(GTK_GRID(tab->audio_grid), tab->audio_agc, 1, 0, 1, 1);
	tab->audio_dcoff = gtk_spin_button_new_with_range(-100, 100, 1);
	gtk_grid_attach(GTK_GRID(tab->audio_grid), tab->audio_dcoff, 1, 2, 1, 1);
}
void create_sync_panel(struct cs_tab *tab){
	tab->sync_frame = gtk_frame_new("Sync Setup: ");
	tab->sync_grid = gtk_grid_new();
	gtk_grid_set_row_spacing(GTK_GRID(tab->sync_grid), 3);
	gtk_grid_attach(GTK_GRID(tab->main_grid), tab->sync_frame, 1, 1, 1, 2);
	gtk_container_add(GTK_CONTAINER(tab->sync_frame), tab->sync_grid);
	tab->line_sync = gtk_check_button_new_with_label("Line Sync");
	gtk_grid_attach(GTK_GRID(tab->sync_grid), tab->line_sync, 0, 0, 1, 1);
	tab->pos_ls = gtk_check_button_new_with_label("Positive Line\nSync");
	gtk_grid_attach(GTK_GRID(tab->sync_grid), tab->pos_ls, 1, 0, 1, 1);
	put_label(tab->sync_grid, "Line Sync Pulse (\xC2\xB5S): ", 0, 1);
	tab->ls_pulse_len = gtk_spin_button_new_with_range(0, 1000, 0.5);
	gtk_grid_attach(GTK_GRID(tab->sync_grid), tab->ls_pulse_len, 1, 1, 1, 1);
	put_label(tab->sync_grid, "Line Sync Amp (% F/S): ", 0, 2);
	tab->ls_pulse_amp = gtk_spin_button_new_with_range(0, 100, 1.0);
	gtk_grid_attach(GTK_GRID(tab->sync_grid), tab->ls_pulse_amp, 1, 2, 1, 1);
	tab->frame_sync = gtk_check_button_new_with_label("Frame Sync");
	gtk_grid_attach(GTK_GRID(tab->sync_grid), tab->frame_sync, 0, 3, 1, 1);
	tab->fs_pulse = gtk_check_button_new_with_label("Frame Sync\nPulse");
	gtk_grid_attach(GTK_GRID(tab->sync_grid), tab->fs_pulse, 1, 3, 1, 1);
	tab->fs_pulse_pos = gtk_check_button_new_with_label("Positive Frame Sync");
	gtk_grid_attach(GTK_GRID(tab->sync_grid), tab->fs_pulse_pos, 0, 4, 2, 1);
	put_label(tab->sync_grid, "Frame Sync Pulse (\xC2\xB5S): ", 0, 5);
	tab->fs_pulse_len = gtk_spin_button_new_with_range(0, 40000, 0.5);
	gtk_grid_attach(GTK_GRID(tab->sync_grid), tab->fs_pulse_len, 1, 5, 1, 1);
	put_label(tab->sync_grid, "Frame Sync Amp (% F/S): ", 0, 6);
	tab->fs_pulse_amp = gtk_spin_button_new_with_range(0, 100, 1.0);
	gtk_grid_attach(GTK_GRID(tab->sync_grid), tab->fs_pulse_amp, 1, 6, 1, 1);
	tab->fs_oll = gtk_check_button_new_with_label("Omit Line Sync\nOn First Line");
	gtk_grid_attach(GTK_GRID(tab->sync_grid), tab->fs_oll, 0, 7, 1, 1);
	tab->fs_ofl = gtk_check_button_new_with_label("Omit Line Sync\nOn Last Line");
	gtk_grid_attach(GTK_GRID(tab->sync_grid), tab->fs_ofl, 1, 7, 1, 1);
	put_label(tab->sync_grid, "Blanking+Sync\nDC Offset (% F/S):", 0, 8);
	tab->sync_dcoff = gtk_spin_button_new_with_range(-100, 100, 1.0);
	gtk_grid_attach(GTK_GRID(tab->sync_grid), tab->sync_dcoff, 1, 8, 1, 1);	
	
}

void invert_channel_changed(GtkWidget *invert, gpointer channel){
	if(c_handler_disable) return;
	nbtv_fmt->channels[(int)channel].invert_channel = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(invert));
	format_changed = 1;
	clrfi();
}
void channel_gain_changed(GtkWidget *gain, gpointer channel){
	if(c_handler_disable) return;
	nbtv_fmt->channels[(int)channel].channel_gain = gtk_range_get_value(GTK_RANGE(gain));
	format_changed = 1;
	clrfi();
}
void video_en(GtkWidget *video, gpointer channel){
	if(c_handler_disable) return;
	nbtv_fmt->channels[(int)channel].video = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(video));
	format_changed = 1;
	clrfi();
	gui_update_channel_setup(nbtv_fmt);
}
void audio_en(GtkWidget *audio, gpointer channel){
	if(c_handler_disable) return;
	if(nbtv_fmt->channels[(int)channel].audio = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(audio))){
		nbtv_fmt->channels[(int)channel].video = 0;
		nbtv_fmt->channels[(int)channel].video = 0;
	}
	format_changed = 1;
	clrfi();
	gui_update_channel_setup(nbtv_fmt);
}
void sync_en(GtkWidget *sync, gpointer channel){
	if(c_handler_disable) return;
	nbtv_fmt->channels[(int)channel].sync = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sync));
	gui_update_channel_setup(nbtv_fmt);
	format_changed = 1;
	clrfi();
}
void invert_video_en(GtkWidget *invert, gpointer channel){
	if(c_handler_disable) return;
	nbtv_fmt->channels[(int)channel].invert_video = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(invert));
	format_changed = 1;
	clrfi();
}
void video_gain_changed(GtkWidget *gain, gpointer channel){
	if(c_handler_disable) return;
	nbtv_fmt->channels[(int)channel].video_level = gtk_spin_button_get_value(GTK_SPIN_BUTTON(gain));
	format_changed = 1;
	clrfi();
}
void cg_changed(GtkWidget *bla, gpointer channel){
	if(c_handler_disable) return;
	struct NBTV_channel *ch = &nbtv_fmt->channels[(int)channel];
	tab = &cs_tabs[(int)channel];
	ch->red_gain = gtk_range_get_value(GTK_RANGE(tab->Rg));
	ch->green_gain = gtk_range_get_value(GTK_RANGE(tab->Gg));
	ch->blue_gain = gtk_range_get_value(GTK_RANGE(tab->Bg));
	format_changed = 1;
	clrfi();
}
void video_dcoff_changed(GtkWidget *dcoff, gpointer channel){
	if(c_handler_disable) return;
	nbtv_fmt->channels[(int)channel].video_dc_off = gtk_spin_button_get_value(GTK_SPIN_BUTTON(dcoff));
	format_changed = 1;
	clrfi();
}
void line_sync_en(GtkWidget *line_sync, gpointer channel){
	if(c_handler_disable) return;
	nbtv_fmt->channels[(int)channel].line_sync = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(line_sync));
	gui_update_channel_setup(nbtv_fmt);
	format_changed = 1;
	clrfi();
}
void line_sync_pos(GtkWidget *ls_pos, gpointer channel){
	if(c_handler_disable) return;
	nbtv_fmt->channels[(int)channel].pos_line_sync = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ls_pos));
	format_changed = 1;
	clrfi();
}
void line_sync_pulse_len(GtkWidget *len, gpointer channel){
	if(c_handler_disable) return;
	nbtv_fmt->channels[(int)channel].ls_length = gtk_spin_button_get_value(GTK_SPIN_BUTTON(len));
	format_changed = 1;
	clrfi();
}
void line_sync_pulse_amp(GtkWidget *amp, gpointer channel){
	if(c_handler_disable) return;
	nbtv_fmt->channels[(int)channel].ls_level = gtk_spin_button_get_value(GTK_SPIN_BUTTON(amp));
	format_changed = 1;
	clrfi();
}

void frame_sync_pulse_len(GtkWidget *len, gpointer channel){
	if(c_handler_disable) return;
	nbtv_fmt->channels[(int)channel].fs_length = gtk_spin_button_get_value(GTK_SPIN_BUTTON(len));
	format_changed = 1;
	clrfi();
}
void frame_sync_pulse_amp(GtkWidget *amp, gpointer channel){
	if(c_handler_disable) return;
	nbtv_fmt->channels[(int)channel].fs_level = gtk_spin_button_get_value(GTK_SPIN_BUTTON(amp));
	format_changed = 1;
	clrfi();
}
void frame_sync_en(GtkWidget *frame_sync, gpointer channel){
	if(c_handler_disable) return;
	nbtv_fmt->channels[(int)channel].frame_sync = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(frame_sync));
	gui_update_channel_setup(nbtv_fmt);
	format_changed = 1;
	clrfi();
}
void frame_sync_pos(GtkWidget *fs_pos, gpointer channel){
	if(c_handler_disable) return;
	nbtv_fmt->channels[(int)channel].pos_frame_sync = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fs_pos));
	format_changed = 1;
	clrfi();
}
void frame_sync_pulse(GtkWidget *fs_pulse, gpointer channel){
	if(c_handler_disable) return;
	nbtv_fmt->channels[(int)channel].fs_pulse = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fs_pulse));
	gui_update_channel_setup(nbtv_fmt);
	format_changed = 1;
	clrfi();
}
void ls_omit(GtkWidget *bla, gpointer channel){
	if(c_handler_disable) return;
	struct NBTV_channel *ch = &nbtv_fmt->channels[(int)channel];
	tab = &cs_tabs[(int)channel];
	ch->fs_no_ls_f = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tab->fs_ofl));
	ch->fs_no_ls_l = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tab->fs_oll));
	format_changed = 1;
	clrfi();
}
void gui_channel_setup_connect_callbacks(){
	int i = 0;
	for(i = 0;i < 4;i++){
		tab = &cs_tabs[i];
		g_signal_connect(tab->channel_invert, "toggled", G_CALLBACK(invert_channel_changed), (gpointer)i);
		g_signal_connect(tab->channel_gain, "value-changed", G_CALLBACK(channel_gain_changed), (gpointer)i);
		g_signal_connect(tab->video_cb, "toggled", G_CALLBACK(video_en), (gpointer)i);
		g_signal_connect(tab->audio_cb, "toggled", G_CALLBACK(audio_en), (gpointer)i);
		g_signal_connect(tab->sync_cb, "toggled", G_CALLBACK(sync_en), (gpointer)i);
		g_signal_connect(tab->invert_video, "toggled", G_CALLBACK(invert_video_en), (gpointer)i);
		g_signal_connect(tab->video_gain, "changed", G_CALLBACK(video_gain_changed), (gpointer)i);
		g_signal_connect(tab->Rg, "value-changed", G_CALLBACK(cg_changed), (gpointer)i);
		g_signal_connect(tab->Gg, "value-changed", G_CALLBACK(cg_changed), (gpointer)i);
		g_signal_connect(tab->Bg, "value-changed", G_CALLBACK(cg_changed), (gpointer)i);
		g_signal_connect(tab->video_dcoff, "value-changed", G_CALLBACK(video_dcoff_changed), (gpointer)i);
		g_signal_connect(tab->line_sync, "toggled", G_CALLBACK(line_sync_en), (gpointer)i);
		g_signal_connect(tab->pos_ls, "toggled", G_CALLBACK(line_sync_pos), (gpointer)i);
		g_signal_connect(tab->ls_pulse_len, "value-changed", G_CALLBACK(line_sync_pulse_len), (gpointer)i);
		g_signal_connect(tab->ls_pulse_amp, "value-changed", G_CALLBACK(line_sync_pulse_amp), (gpointer)i);
		g_signal_connect(tab->frame_sync, "toggled", G_CALLBACK(frame_sync_en), (gpointer)i);
		g_signal_connect(tab->fs_pulse_pos, "toggled", G_CALLBACK(frame_sync_pos), (gpointer)i);
		g_signal_connect(tab->fs_pulse, "toggled", G_CALLBACK(frame_sync_pulse), (gpointer)i);
		g_signal_connect(tab->fs_pulse_len, "value-changed", G_CALLBACK(frame_sync_pulse_len), (gpointer)i);
		g_signal_connect(tab->fs_pulse_amp, "value-changed", G_CALLBACK(frame_sync_pulse_amp), (gpointer)i);
		g_signal_connect(tab->fs_oll, "toggled", G_CALLBACK(ls_omit), (gpointer)i);
		g_signal_connect(tab->fs_ofl, "toggled", G_CALLBACK(ls_omit), (gpointer)i);
		
	}
}
GtkWidget *create_channel_setup_window(){
	cs_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	g_signal_connect(cs_window, "destroy", G_CALLBACK(cs_window_close), NULL);
	gtk_window_set_title(GTK_WINDOW(cs_window), "Channel Setup");
	gtk_window_set_resizable(GTK_WINDOW(cs_window), FALSE);
	tabby = gtk_notebook_new();
	gtk_container_add(GTK_CONTAINER(cs_window), tabby);
	char buffer[32];
	int i = 0;
	for(i = 0;i < 4;i++){
		tab = &cs_tabs[i];
		tab->main_grid = gtk_grid_new();
		gtk_grid_set_column_spacing(GTK_GRID(tab->main_grid), 4);
		sprintf(buffer, "Channel %d", i);
		gtk_notebook_append_page(GTK_NOTEBOOK(tabby), tab->main_grid, gtk_label_new(buffer));
		create_selection_panel(tab);
		create_video_panel(tab);
		create_audio_panel(tab);
		create_sync_panel(tab);

	}
	gtk_widget_show_all(cs_window);
	gui_channel_setup_connect_callbacks();

}